--- computations for construction A, B and C applied to polygon of type 5a
--- computations for construction B applied to polygon of type 5a up to dimension eight

needsPackage "Polyhedra"

-- lattice points of construction A(P), B(P) and C(P) where P is the polygon of type 5a
AA = matrix{{0,0,0,-1,-1,-1,0,0,0,1,1,1,-1,-1,-1,0,0,0},{-1,-1,-1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1},{-1,0,1,-1,0,1,-1,0,1,-1,0,1,-1,0,1,-1,0,1}}; -- A(P) contruction
AB = matrix{{0,-1,0,1,-1,0,0,0},{-1,0,0,0,1,1,0,0},{0,0,0,0,0,0,1,-1}}; -- B(P) contruction
AC = matrix{{0,-1,0,1,-1,0,0,-1,0,1,-1,0,0},{-1,0,0,0,1,1,-1,0,0,0,1,1,0},{-1,-1,-1,-1,-1,-1,0,0,0,0,0,0,1}}; -- C(P) construction
-- lattice points of B^k(P) up to dimension eight
AB4d = matrix{{0,-1,0,-1,1,0,0,0,0,0},{0,1,1,0,0,-1,0,0,0,0},{0,0,0,0,0,0,1,-1,0,0},{0,0,0,0,0,0,0,0,1,-1}};
AB5d = matrix{{0,-1,0,-1,1,0,0,0,0,0,0,0},{0,1,1,0,0,-1,0,0,0,0,0,0},{0,0,0,0,0,0,1,-1,0,0,0,0},{0,0,0,0,0,0,0,0,1,-1,0,0},{0,0,0,0,0,0,0,0,0,0,1,-1}};
AB6d = matrix{{0,-1,0,-1,1,0,0,0,0,0,0,0,0,0},{0,1,1,0,0,-1,0,0,0,0,0,0,0,0},{0,0,0,0,0,0,1,-1,0,0,0,0,0,0},{0,0,0,0,0,0,0,0,1,-1,0,0,0,0},{0,0,0,0,0,0,0,0,0,0,1,-1,0,0},{0,0,0,0,0,0,0,0,0,0,0,0,1,-1}};
AB7d = matrix{{0,-1,0,-1,1,0,0,0,0,0,0,0,0,0,0,0},{0,1,1,0,0,-1,0,0,0,0,0,0,0,0,0,0},{0,0,0,0,0,0,1,-1,0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0,1,-1,0,0,0,0,0,0},{0,0,0,0,0,0,0,0,0,0,1,-1,0,0,0,0},{0,0,0,0,0,0,0,0,0,0,0,0,1,-1,0,0},{0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,-1}};
AB8d = matrix{{0,-1,0,-1,1,0,0,0,0,0,0,0,0,0,0,0,0,0},{0,1,1,0,0,-1,0,0,0,0,0,0,0,0,0,0,0,0},{0,0,0,0,0,0,1,-1,0,0,0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0,1,-1,0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0,0,0,1,-1,0,0,0,0,0,0},{0,0,0,0,0,0,0,0,0,0,0,0,1,-1,0,0,0,0},{0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,-1,0,0},{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,-1}};

-- test reflexive (example)
P = convexHull AA
isReflexive P

-- determine matrix A with all-1-vector and nonnegative entries
matrixA = i -> (
    P = convexHull i;
    L = latticePoints P;
    L = matrix{L};
    n = numgens source L;
    d = numgens target L;
    seqn = 0..n-1;
    seqd = 0..d-1;
    listn = toList seqn;
    listd = toList seqd;
    listd = apply(listd, i -> min apply(listn, j -> L_(i,j)));
    T = matrix table(d,n,(i,j) -> -listd_i);
    L = L + T;
    E = transpose matrix vector(apply(listn, i -> 1));
    L = E||L
)

AA = matrixA AA
AB = matrixA AB
AC = matrixA AC

AB4d = matrixA AB4d
AB5d = matrixA AB5d
AB6d = matrixA AB6d
AB7d = matrixA AB7d
AB8d = matrixA AB8d


-- write julia file for homotopy continuation
-- determine ml equations
MLeqs = {coeffRing=>QQ} >> o -> (expr) -> (
    m = numgens source expr;
    R := (o.coeffRing)[vars {53..(52+(numgens target expr))},u_1..u_m];
    A := substitute(expr,R);
    U = u_1..u_m;
    U = toList U;
    U = transpose matrix {for i in U list i_R};
    N = sum for i from 1 to m list u_i;
    X := for i in 0..(m-1) list R_(entries expr_i);
    X = transpose matrix {X};
    system := ideal (A*((N * X) - U));
    return first entries gens system
)

AAPolynomials = MLeqs AA
ABPolynomials = MLeqs AB
ACPolynomials = MLeqs AC

AB4dPolynomials = MLeqs AB4d
AB5dPolynomials = MLeqs AB5d
AB6dPolynomials = MLeqs AB6d
AB7dPolynomials = MLeqs AB7d
AB8dPolynomials = MLeqs AB8d


polynomialjl = p -> (
    replace("_1","[1]",replace("_2","[2]",replace("_3","[3]",replace("_4","[4]",replace("_5","[5]",replace("_6","[6]",
    replace("_7","[7]",replace("_8","[8]",replace("_9","[9]",replace("_10","[10]",replace("_11","[11]",
    replace("_12","[12]",replace("_13","[13]",replace("_14","[14]",replace("_15","[15]",replace("_16","[16]",replace("_17","[17]",replace("_18","[18]",toExternalString p))))))))))))))))))
)

writeSystem = method(Options=>{})

writeSystem (List, File) := o -> (P,f) -> (
    variables = {"x1","x2","x3","x4","x5","x6","x7","x8","x9","u[1:18]"};
    f << "## This file was generated by Macaulay2 to compute the ML degree via homotopy continuation.\n\n";
    f << "using HomotopyContinuation";
    f << "\n\n";
    f << "@var " | concatenate(mingle(variables, #variables:1));
    eqnCommas:=(#P-1):", ";
    f << "\n\n";
    f << "F = System(\n";
    f << "    [\n";
    f << "         "|concatenate mingle(apply(P,e->polynomialjl e), eqnCommas);
    f << "\n";
    f << "    ];\n";
    f << "    parameters = u\n"  ;
    f << ")\n\n";
    f << "## Solve generically:\n\n";
    f << "u0 = rand(ComplexF64,18)\n";
    f << "result_u0 = solve(F, target_parameters = u0)\n\n";
    f << "## Solve for random parameter values\n\n";
    f << "data = [rand((0:100000),18)]\n";
    f << "data_points = solve(\n";
    f << "      F,\n";
    f << "      solutions(result_u0),\n";
    f << "      start_parameters =  u0,\n";
    f << "      target_parameters = data,\n";
    f << ")";
)

writeSystem (List, String) :=o -> (P,filename) -> (
    f := openOut filename;
    writeSystem(P,f);
    close f;
)

writeSystem(AAPolynomials, "5aConstructionA.jl")
writeSystem(ABPolynomials, "5aConstructionB.jl")
writeSystem(ACPolynomials, "5aConstructionC.jl")

writeSystem(AB4dPolynomials, "5aConstructionB_4D.jl")
writeSystem(AB5dPolynomials, "5aConstructionB_5D.jl")
writeSystem(AB6dPolynomials, "5aConstructionB_6D.jl")
writeSystem(AB7dPolynomials, "5aConstructionB_7D.jl")
writeSystem(AB8dPolynomials, "5aConstructionB_8D.jl")